class ElectionController < ApplicationController
  require_dependency "raw_voter_list"
  require_dependency "voter"
  require_dependency "vote"
  require_dependency "candidate"
  layout 'main'

  ## methods for displaying, creating,
  ## and manipulating election overview data
  ####################################################################

  def new
    redirect_to :action => 'general_information'
  end
  
  def general_information
    @sidebar_content = render_to_string :partial => 'progress',
                                        :locals => { :page => 'overview' }
    @election = Election.new
    render :action => 'general_information'
  end
  
  def create_election
    @election = Election.new(params[:election])
    
    # default options
    @election.user = session[:user]
    @election.anonymous = 1
    @election.startdate = Time.now

    if @election.save
      flash[:notice] = 'Election was successfully created.'
      redirect_to :action => 'edit_candidates', :id => @election.id
    else
      render :action => 'general_information'
    end
  end
  
  # add filter to verify that the person working on or looking at
  # something is the owner
  def edit
    @election = Election.find(params[:id])
  end

  def show
    @election = Election.find(params[:id])
  end

  def update
    @election = Election.find(params[:id])
    if @election.update_attributes(params[:election])
      flash[:notice] = 'Election was successfully updated.'
      redirect_to :action => 'show', :id => @election
    else
      render :action => 'edit'
    end
  end

  def start_election
    @election = Election.find(params[:id])
    @election.voters.each do |voter|
      voter.vote = Vote.new
      email_voter voter
    end

    @election.activate!
    redirect_to :action => 'show', :id => @election.id
  end

  # methods fod display, adding, deleting, and manipulating candidate
  # information for elections
  ####################################################################
  def edit_candidates
    @sidebar_content = render_to_string :partial => 'progress',
                                        :locals => { :page => 'candidates' }
    @election = Election.find( params[:id] )
  end

  def add_candidate
    @election = Election.find(params[:id])
    @candidate = Candidate.new(params[:candidate])
    @election.candidates << @candidate

    if @candidate.save
      @candidate = Candidate.new
      redirect_to :action => 'edit_candidates', :id => @election.id
    else
      render :action => 'edit_candidates', :id => @election.id
    end
  end
  
  def delete_candidate
    candidate = Candidate.find( params[:id] )
    candidate.destroy
  end

  def lessinfo_candidate
    @show_details = false
    @current_candidate = Candidate.find( params[:id] )
    render :partial => 'candidate_line'
  end

  def moreinfo_candidate
    @show_details = true
    @current_candidate = Candidate.find( params[:id] )
    render :partial => 'candidate_line'
  end

  def edit_candidate
    @candidate = Candidate.find( params[:id] )
    @election = @candidate.election
  end

  def update_candidate
    @candidate = Candidate.find(params[:id])
    @election = @candidate.election

    if @candidate.update_attributes(params[:candidate])
      redirect_to :action => 'edit_candidates', :id => @candidate.election.id
    else
      render :action => 'edit_candidate'
    end
  end

  def candidate_picture
    candidate = Candidate.find( params[:id] )
    send_data( candidate.picture.data,
               :filename => candidate.picture.filename,
	       :type => candidate.picture.filetype,
	       :disposition => 'inline' )
  end

  ## methods for displaying, adding, deleting, and manipulating voters
  ## for a particular election
  ####################################################################
  def new_voters
    edit_voters
  end
  
  def edit_voters
    @election = Election.find( params[:id] )
    if params.has_key?( :raw_voter_list )
      process_incoming_voters( params[:raw_voter_list] )
    end
    @raw_voter_list = RawVoterList.new
  end
  
  def delete_voter
    voter = Voter.find( params[:id] )
    voter.destroy
  end
  
  def remind_voter
    voter_array= FullVoter.find(:all, :conditions => ["email = ?", params[:email]])
    voter_array.delete_if {|voter| voter.election.active == 0}
    unless voter_array.empty?
      VoterNotify.deliver_reminder(voter_array)
    end
  end
  
  ## methods for computing and printing results
  ####################################################################
  def results
    @election = Election.find( params[:id] )
    votes = []

    @election.voters.each do |voter|
      if voter.vote and voter.vote.confirmed?
        votes << voter.vote.rankings.sort.collect {|vote| vote.candidate_id}
      end
    end
    
    @voteobj = CloneproofSSDVote.new(votes)
    @resultobj = @voteobj.result
    @winners = @resultobj.winners
    
    @candidates_by_id = {}
    @election.candidates.each {|cand| @candidates_by_id[cand.id] = cand}
  end
  
  def detailed_results
   
    self.results

    @voter_list = []
    @vote_list = []
    @election.voters. each do |voter|
      if voter.vote and voter.vote.confirmed?
        @voter_list << voter.email
	      @vote_list << voter.vote
      end
    end

    @vote_list.sort!
    @vote_list.sort! { |a,b| a.token <=> b.token }
  end

  ## private methods
  ####################################################################
  private

    def process_incoming_voters(raw_voter_list)
      incoming_voters = RawVoterList.new( raw_voter_list )

      unless incoming_voters.entries.empty?
        incoming_voters.each do |new_voter|
          new_voter.email.strip! # There's a trailing \r on all but the last in
                                 # the list!
          if new_voter.valid?
            # the new voter should be in good shape. save add to the election
            @election.voters << new_voter
	          new_voter.save
          end
          # TODO: Can we do some kind of AJAX error message for the voter being invalid?
        end
        @election.save
      end
 
      # reset the next time to have a the same default value for emailing
      @raw_voter_list = RawVoterList.new
      @raw_voter_list.email = incoming_voters.email
    end

    def email_voter(voter=nil)
      if voter
        VoterNotify.deliver_votestart(voter)
        voter.contacted=1
        voter.save
      end
    end

end
